/**
 * @internal
 */
function hthvItem({ $ = 'raw', n, t, v, x = [], p = {}, pl = [], }) {
    if (n) {
        p[n] = { $, n, v, x: [], p: {}, pl: [] };
    }
    return {
        $,
        n,
        t,
        v,
        x,
        p,
        pl,
    };
}

/**
 * @internal
 */
function addParam({ p, pl }, param) {
    const key = param.n || param.v;
    const prev = p[key];
    // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
    if (!prev || (!prev.n && param.n)) {
        p[key] = param;
    }
    pl.push(param);
}

/**
 * Constants for various kinds of delimiters.
 *
 * Each value is a bit mask that can be used to check whether the given delimiter kind corresponds to it.
 */
var HthvDelimiter;
(function (HthvDelimiter) {
    /**
     * Not a delimiter.
     *
     * Corresponds to any non-delimiting character.
     */
    HthvDelimiter[HthvDelimiter["None"] = 0] = "None";
    /**
     * A delimiter that can not be included into toke.
     *
     * Any {@link HthvDelimiterChar delimiter character} considered non-token by default.
     */
    HthvDelimiter[HthvDelimiter["NonToken"] = 1] = "NonToken";
    /**
     * Space character.
     *
     * This is either space (U+0020), or horizontal tab (U+0009) characters.
     */
    HthvDelimiter[HthvDelimiter["Space"] = 2] = "Space";
    /**
     * Items delimiter.
     *
     * This is either a {@link Space space character}, or colon (U+002C) by default.
     */
    HthvDelimiter[HthvDelimiter["Item"] = 4] = "Item";
    /**
     * Parameters delimiter.
     *
     * This is a `;` sign by default.
     */
    HthvDelimiter[HthvDelimiter["Parameter"] = 8] = "Parameter";
    /**
     * An assignment of value to some name.
     *
     * A delimiter between name token and value.
     *
     * This is a `=` sign by default. Or `:` sign in comments.
     */
    HthvDelimiter[HthvDelimiter["Assignment"] = 16] = "Assignment";
    /**
     * A delimiter that should be escaped with a backslash.
     */
    HthvDelimiter[HthvDelimiter["Escaped"] = 32] = "Escaped";
    /**
     * A quoted-string delimiter.
     *
     * This is a `"` sign by default.
     */
    HthvDelimiter[HthvDelimiter["Quote"] = 65536] = "Quote";
    /**
     * Delimiter starting a comment.
     *
     * This is a '(' sign when comments enabled.
     */
    HthvDelimiter[HthvDelimiter["Comment"] = 131072] = "Comment";
    /**
     * Delimiter staring and angle-bracketed-string.
     *
     * This is a `<` sign by default.
     */
    HthvDelimiter[HthvDelimiter["AngleBracketStart"] = 262144] = "AngleBracketStart";
    /**
     * Delimiter staring and angle-bracketed-string.
     *
     * This is a `>` sign by default.
     */
    HthvDelimiter[HthvDelimiter["AngleBracketEnd"] = 524288] = "AngleBracketEnd";
})(HthvDelimiter || (HthvDelimiter = {}));

/**
 * @internal
 */
function angleBracketsParser({ delimiterOf, }) {
    return (input, out) => {
        if (!(delimiterOf(input.s[input.i]) & HthvDelimiter.AngleBracketStart)) {
            return false;
        }
        let result = '';
        ++input.i;
        while (input.i < input.s.length) {
            const c = input.s[input.i++];
            if (delimiterOf(c) & HthvDelimiter.AngleBracketEnd) {
                break;
            }
            result += c;
        }
        out(result);
        return true;
    };
}

/**
 * @internal
 */
const parseNone = () => false;

const datePattern = /^(Mon|Tue|Wed|Thu|Fri|Sat|Sun), \d\d (Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec) \d\d\d\d \d\d:\d\d:\d\d GMT/;
/**
 * @internal
 */
function dateTimeParser({ dateTime, }) {
    return dateTime ? parseDateTime : parseNone;
}
function parseDateTime(input, out) {
    input.s = input.s.substring(input.i);
    input.i = 0;
    if (datePattern.test(input.s)) {
        out(input.s.substring(input.i, (input.i += 29)));
        return true;
    }
    return false;
}

/**
 * @internal
 */
function nextInItem({ delimiterOf }) {
    return input => {
        const c = input.s[input.i];
        input.d = delimiterOf(c);
        return c;
    };
}

/**
 * @internal
 */
function quotedStringParser({ delimiterOf, }) {
    return (input, out) => {
        let unquoted = '';
        ++input.i;
        for (; input.i < input.s.length; ++input.i) {
            const c = input.s[input.i];
            if (c === '\\') {
                const next = input.s[++input.i];
                if (next) {
                    unquoted += next;
                }
                else {
                    unquoted += c;
                }
            }
            else if (delimiterOf(c) & HthvDelimiter.Quote) {
                ++input.i;
                out(unquoted);
                return;
            }
            else {
                unquoted += c;
            }
        }
        out(unquoted);
    };
}

/**
 * @internal
 */
function itemParser(config, { named = true, tagged = true, extra = true, next = nextInItem(config) } = {}) {
    const parseAngleBrackets = angleBracketsParser(config);
    const parseDateTime = dateTimeParser(config);
    const parseQuotedString = quotedStringParser(config);
    const parseExtra = extra
        ? itemParser(config, { next, tagged: false, named: false, extra: false })
        : parseNone;
    return (input, out) => {
        let name = '';
        let type = 'raw';
        let tag;
        let value;
        while (input.i < input.s.length) {
            const c = next(input);
            if (input.d) {
                if (input.d & (HthvDelimiter.Item | HthvDelimiter.Parameter)) {
                    break;
                }
                if (value == null) {
                    if (input.d & HthvDelimiter.Assignment) {
                        value = name ? '' : c;
                        ++input.i;
                        continue;
                    }
                    if (input.d & HthvDelimiter.Quote) {
                        if (tagged || !name) {
                            parseQuotedString(input, v => {
                                if (name) {
                                    type = 'tagged-string';
                                    tag = name;
                                }
                                else {
                                    type = 'quoted-string';
                                }
                                name = '';
                                value = v;
                            });
                        }
                        break;
                    }
                    if (!name
                        && parseAngleBrackets(input, v => {
                            type = 'angle-bracketed-string';
                            value = v;
                        })) {
                        break;
                    }
                    value = name;
                    name = '';
                }
                else if (input.d & HthvDelimiter.Quote) {
                    if (tagged || !value) {
                        parseQuotedString(input, v => {
                            if (value) {
                                type = 'tagged-string';
                                tag = value;
                            }
                            else {
                                type = 'quoted-string';
                            }
                            value = v;
                        });
                    }
                    break;
                }
                else if (!value
                    && parseAngleBrackets(input, v => {
                        type = 'angle-bracketed-string';
                        value = v;
                    })) {
                    break;
                }
            }
            if (value == null) {
                if (!name && parseDateTime(input, v => (value = v))) {
                    type = 'date-time';
                    break;
                }
                if (named) {
                    name += c;
                }
                else {
                    value = c;
                }
            }
            else if (!value && parseDateTime(input, v => (value = v))) {
                type = 'date-time';
                break;
            }
            else {
                value += c;
            }
            ++input.i;
        }
        let item;
        if (value == null) {
            if (!name) {
                return false;
            }
            item = hthvItem({ $: type, v: name });
        }
        else {
            item = hthvItem({ $: type, n: name || undefined, t: tag, v: value });
        }
        // noinspection StatementWithEmptyBodyJS
        while (parseExtra(input, extraItem => item.x.push(extraItem)))
            ; // eslint-disable-line curly
        out(item);
        return true;
    };
}

/**
 * @internal
 */
function nextInComment({ delimiterOf }) {
    return input => {
        let c = input.s[input.i];
        if (c !== '\\') {
            input.d = delimiterOf(c);
            return c;
        }
        ++input.i;
        if (input.i < input.s.length) {
            c = input.s[input.i];
            input.d = delimiterOf(c) ? HthvDelimiter.NonToken : HthvDelimiter.None;
        }
        else {
            input.d = HthvDelimiter.NonToken;
        }
        return c;
    };
}

/**
 * @internal
 */
function spacesParser({ delimiterOf }) {
    return input => {
        const start = input.i;
        do {
            const c = input.s[input.i];
            if (!(delimiterOf(c) & HthvDelimiter.Space)) {
                break;
            }
            input.i++;
        } while (input.i < input.s.length);
        return input.i !== start;
    };
}

/**
 * @internal
 */
function paramParser(config, opts = {}) {
    const { delimiterOf } = config;
    const skipSpaces = spacesParser(config);
    const parseItem = itemParser(config, { ...opts, tagged: false });
    return (input, out) => {
        if (!(delimiterOf(input.s[input.i]) & HthvDelimiter.Parameter)) {
            return false;
        }
        ++input.i;
        skipSpaces(input);
        return parseItem(input, out);
    };
}

/**
 * @internal
 */
function commentParser(config) {
    const { delimiterOf } = config;
    const skipSpaces = spacesParser(config);
    const commentParserConfig = {
        next: nextInComment(config),
    };
    const parseItem = itemParser(config, commentParserConfig);
    const parseParam = paramParser(config, commentParserConfig);
    return (input, out) => {
        if (!(delimiterOf(input.s[input.i]) & HthvDelimiter.Comment)) {
            return false;
        }
        ++input.i;
        let result;
        // noinspection StatementWithEmptyBodyJS
        while (skipSpaces(input)
            || parseParam(input, param => {
                if (!result) {
                    result = hthvItem({ $: 'raw', v: '' });
                }
                addParam(result, param);
            })
            || parseItem(input, item => (result = item)))
            ; // eslint-disable-line curly
        ++input.i; // closing parent
        out(result || hthvItem({ $: 'raw', v: '' }));
        return true;
    };
}

/**
 * @internal
 */
function itemDelimitParser(config) {
    const { delimiterOf } = config;
    const skipSpaces = spacesParser(config);
    return input => {
        if (delimiterOf(input.s[input.i]) & HthvDelimiter.Item) {
            input.i++;
            skipSpaces(input);
            return true;
        }
        return false;
    };
}

/**
 * @internal
 */
const defaultDelimit = {
    ' ': HthvDelimiter.NonToken | HthvDelimiter.Space | HthvDelimiter.Item,
    '\t': HthvDelimiter.NonToken | HthvDelimiter.Space | HthvDelimiter.Item,
    ',': HthvDelimiter.NonToken | HthvDelimiter.Item,
    ';': HthvDelimiter.NonToken | HthvDelimiter.Parameter,
    '"': HthvDelimiter.NonToken | HthvDelimiter.Escaped | HthvDelimiter.Quote,
    '\\': HthvDelimiter.NonToken | HthvDelimiter.Escaped,
    '<': HthvDelimiter.NonToken | HthvDelimiter.AngleBracketStart,
    '>': HthvDelimiter.NonToken | HthvDelimiter.AngleBracketEnd,
    '=': HthvDelimiter.NonToken | HthvDelimiter.Assignment,
    '(': HthvDelimiter.NonToken,
    ')': HthvDelimiter.NonToken,
    '/': HthvDelimiter.NonToken,
    ':': HthvDelimiter.NonToken,
    '?': HthvDelimiter.NonToken,
    '@': HthvDelimiter.NonToken,
    '[': HthvDelimiter.NonToken,
    ']': HthvDelimiter.NonToken,
    '{': HthvDelimiter.NonToken,
    '}': HthvDelimiter.NonToken,
};
/**
 * @internal
 */
function buildParserConfig({ delimit, dateTime } = {}) {
    const delimitConfig = delimit ? { ...defaultDelimit, ...delimit } : defaultDelimit;
    return {
        dateTime,
        delimiterOf(c) {
            return (delimitConfig[c]
                || ((c >= '\u0000' && c <= ' ') || c === '\u007f'
                    ? HthvDelimiter.NonToken
                    : HthvDelimiter.None));
        },
    };
}
/**
 * @internal
 */
const defaultParserConfig = /*#__PURE__*/ buildParserConfig();
/**
 * @internal
 */
const commentParserConfig = /*#__PURE__*/ buildParserConfig({
    delimit: {
        ':': HthvDelimiter.NonToken | HthvDelimiter.Assignment,
        '(': HthvDelimiter.NonToken | HthvDelimiter.Escaped | HthvDelimiter.Comment,
        ')': HthvDelimiter.NonToken | HthvDelimiter.Escaped | HthvDelimiter.Item,
        ' ': HthvDelimiter.NonToken | HthvDelimiter.Space,
        '\t': HthvDelimiter.NonToken | HthvDelimiter.Space,
        '=': HthvDelimiter.NonToken,
        ',': HthvDelimiter.NonToken,
    },
});

/**
 * Creates and configures new HTTP header value parser.
 *
 * @param config - New parser configuration.
 *
 * @returns New HTTP header value parser function.
 */
function newHthvParser(config) {
    const parserConfig = config ? buildParserConfig(config) : defaultParserConfig;
    const parseItemDelimit = itemDelimitParser(parserConfig);
    const parseParam = paramParser(parserConfig);
    const parseItem = itemParser(parserConfig);
    const parseComment = (config === null || config === void 0 ? void 0 : config.comments) ? commentParser(commentParserConfig) : parseNone;
    return headerValue => {
        const result = [];
        const input = { i: 0, s: headerValue };
        // noinspection StatementWithEmptyBodyJS
        while (parseTopLevelItem())
            ; // eslint-disable-line curly
        return result;
        function parseTopLevelItem() {
            return (input.i < input.s.length
                && (parseItemDelimit(input)
                    || parseParam(input, param => {
                        if (!result.length) {
                            result.push(hthvItem({ $: 'raw', v: '' }));
                        }
                        addParam(result[result.length - 1], param);
                    })
                    || parseComment(input, item => result.push(item))
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-argument
                    || parseItem(input, item => result.push(item))));
        }
    };
}

/**
 * Parses HTTP header value.
 *
 * Splits the value onto {@link HthvItem items}.
 *
 * Parser is configured with default {@link HthvParserConfig config}.
 *
 * Does not recognize comments or date/time values.
 *
 * Treats illegal characters as ASCII letters.
 *
 * @param value - HTTP header value to parse.
 *
 * @returns An array of comma- or space- separated value items.
 */
const hthvParse = /*#__PURE__*/ newHthvParser();
/**
 * Parses HTTP header value that may contain date/time.
 *
 * Splits the value onto {@link HthvItem items}.
 *
 * Parser is configured with default config except {@link HthvParserConfig.dateTime date/time parsing enabled}.
 *
 * Does not recognize comments.
 *
 * Handles date/time values in [IMF-fixdate] format only.
 *
 * Treats illegal characters as ASCII letters.
 *
 * [IMF-fixdate]: https://tools.ietf.org/html/rfc7231#section-7.1.1.1
 *
 * @param value - HTTP header value to parse.
 *
 * @returns An array of comma- or space- separated value items.
 */
const hthvParseDT = /*#__PURE__*/ newHthvParser({ dateTime: true });

export { HthvDelimiter as H, hthvParseDT as a, hthvItem as b, commentParserConfig as c, defaultParserConfig as d, hthvParse as h, newHthvParser as n };
//# sourceMappingURL=hthv-parse-89a10775.js.map
