import { b as hthvItem, h as hthvParse } from './hthv-parse-89a10775.js';

/**
 * @internal
 */
function hthvParseTrivial(value) {
    if (typeof value === 'string') {
        return value.split(',').map(item => item.trim());
    }
    const result = [];
    for (const val of value) {
        result.push(...hthvParseTrivial(val));
    }
    return result;
}
/**
 * @internal
 */
function hthvParseFirstTrivial(value) {
    if (!value) {
        return value;
    }
    if (typeof value === 'string') {
        const commaIdx = value.indexOf(',');
        return (commaIdx >= 0 ? value.substr(0, commaIdx) : value).trim();
    }
    return hthvParseFirstTrivial(value[Symbol.iterator]().next().value);
}

const HttpForwardTrust = {
    /**
     * Builds HTTP request forwarding trust checker function by forwarding info trust policy.
     *
     * @param trust - A trust policy to proxy forwarding records.
     *
     * @returns Constructed trust predicate function.
     */
    by(trust = {}) {
        const { trusted = false } = trust;
        if (typeof trusted === 'function') {
            return trusted; // Already a function.
        }
        if (typeof trusted === 'boolean') {
            const mask = trusted ? 3 /* HttpForwardTrustMask.AlwaysTrust */ : 0 /* HttpForwardTrustMask.DontTrust */;
            return () => mask;
        }
        const params = trusted;
        return item => {
            let result = itemTrustMask(item);
            for (const p of item.pl) {
                result |= itemTrustMask(p);
            }
            return result;
        };
        function itemTrustMask({ n, v }) {
            var _a;
            return (n && ((_a = params[n]) === null || _a === void 0 ? void 0 : _a[v])) || 0 /* HttpForwardTrustMask.DontTrust */;
        }
    },
};

const HttpForwardRep = {
    /**
     * Builds trusted proxy forwarding report by proxy forwarding records.
     *
     * @param items - `Forwarded` header value items.
     * @param defaults - Forwarding info defaults.
     * @param trust - A trust policy to proxy forwarding records.
     *
     * @returns Trusted proxy forwarding report.
     */
    build(items, defaults, trust = {}) {
        var _a, _b, _c, _d;
        const trustChecker = HttpForwardTrust.by(trust);
        let { by: trustedBy, for: trustedFor, host: trustedHost, proto: trustedProto } = defaults;
        const lastHost = hthvItem({ n: 'host', v: trustedHost });
        const lastProto = hthvItem({ n: 'proto', v: trustedProto });
        let trusted = hthvItem({
            n: 'for',
            v: trustedFor,
            p: { host: lastHost, proto: lastProto },
            pl: [lastHost, lastProto],
        });
        // eslint-disable-next-line @typescript-eslint/no-unsafe-argument
        let trustedFlag = trustChecker(trusted, 0); // The last item is always trusted
        let index = 0;
        for (let i = items.length - 1; i >= 0; --i) {
            const item = items[i];
            // Either explicitly trusted or trusted by previous item.
            trustedFlag =
                trustChecker(item, ++index) | ((trustedFlag & 2 /* HttpForwardTrustMask.TrustPrevious */) >>> 1);
            if (!(trustedFlag & 1 /* HttpForwardTrustMask.TrustCurrent */)) {
                break; // Current record is not trusted even though the previous could be trusted here.
            }
            const { p } = item;
            trustedBy = ((_a = p.by) === null || _a === void 0 ? void 0 : _a.v) || trustedBy;
            trustedFor = ((_b = p.for) === null || _b === void 0 ? void 0 : _b.v) || trustedFor;
            trustedHost = ((_c = p.host) === null || _c === void 0 ? void 0 : _c.v) || trustedHost;
            trustedProto = ((_d = p.proto) === null || _d === void 0 ? void 0 : _d.v) || trustedProto;
            trusted = item;
        }
        const result = {};
        for (const { n, v } of trusted.pl) {
            if (n) {
                result[n] = v;
            }
        }
        if (trusted.n) {
            result[trusted.n] = trusted.v;
        }
        // Fill the required fields
        result.by = trustedBy;
        result.for = trustedFor;
        result.host = trustedHost;
        result.proto = trustedProto;
        return result;
    },
    /**
     * Builds trusted proxy forwarding report by HTTP request.
     *
     * @param headers - Request headers.
     * @param defaults - Forwarding info defaults.
     * @param trust - A trust policy to proxy forwarding records.
     *
     * @returns Trusted proxy forwarding report.
     */
    by(headers, defaults, trust = {}) {
        const { forwarded } = headers;
        let items;
        if (forwarded) {
            if (typeof forwarded === 'string') {
                items = hthvParse(forwarded);
            }
            else {
                items = [];
                for (const fwd of forwarded) {
                    items.push(...hthvParse(fwd));
                }
            }
        }
        else if (trust.xForwarded === false) {
            return { ...defaults };
        }
        else {
            items = hthvXForwardedItems(headers);
        }
        return HttpForwardRep.build(items, defaults, trust);
    },
};
/**
 * @internal
 */
function hthvXForwardedItems(headers) {
    const forwardedForValue = headers['x-forwarded-for'];
    if (!forwardedForValue) {
        return [];
    }
    const forwardedFor = hthvParseTrivial(forwardedForValue);
    const forwardedHost = hthvParseFirstTrivial(headers['x-forwarded-host']);
    const forwardedProto = hthvParseFirstTrivial(headers['x-forwarded-proto']);
    const host = forwardedHost && hthvItem({ n: 'host', v: forwardedHost });
    const proto = forwardedProto && hthvItem({ n: 'proto', v: forwardedProto });
    const items = [];
    for (const v of forwardedFor) {
        const p = {};
        const pl = [];
        if (host) {
            p.host = host;
            // eslint-disable-next-line @typescript-eslint/no-unsafe-argument
            pl.push(host);
        }
        if (proto) {
            p.proto = proto;
            // eslint-disable-next-line @typescript-eslint/no-unsafe-argument
            pl.push(proto);
        }
        items.push(hthvItem({ n: 'for', v, p, pl }));
    }
    return items;
}

export { HttpForwardRep as H, HttpForwardTrust as a };
//# sourceMappingURL=http-forward-rep-51e650c0.js.map
