/// <reference lib="es2019" />
declare module "@hatsy/http-header-value" {
    /**
     * Constants for various kinds of delimiters.
     *
     * Each value is a bit mask that can be used to check whether the given delimiter kind corresponds to it.
     */
    export enum HthvDelimiter {
        /**
         * Not a delimiter.
         *
         * Corresponds to any non-delimiting character.
         */
        None = 0,
        /**
         * A delimiter that can not be included into toke.
         *
         * Any {@link HthvDelimiterChar delimiter character} considered non-token by default.
         */
        NonToken = 1,
        /**
         * Space character.
         *
         * This is either space (U+0020), or horizontal tab (U+0009) characters.
         */
        Space = 2,
        /**
         * Items delimiter.
         *
         * This is either a {@link Space space character}, or colon (U+002C) by default.
         */
        Item = 4,
        /**
         * Parameters delimiter.
         *
         * This is a `;` sign by default.
         */
        Parameter = 8,
        /**
         * An assignment of value to some name.
         *
         * A delimiter between name token and value.
         *
         * This is a `=` sign by default. Or `:` sign in comments.
         */
        Assignment = 16,
        /**
         * A delimiter that should be escaped with a backslash.
         */
        Escaped = 32,
        /**
         * A quoted-string delimiter.
         *
         * This is a `"` sign by default.
         */
        Quote = 65536,
        /**
         * Delimiter starting a comment.
         *
         * This is a '(' sign when comments enabled.
         */
        Comment = 131072,
        /**
         * Delimiter staring and angle-bracketed-string.
         *
         * This is a `<` sign by default.
         */
        AngleBracketStart = 262144,
        /**
         * Delimiter staring and angle-bracketed-string.
         *
         * This is a `>` sign by default.
         */
        AngleBracketEnd = 524288
    }
    /**
     * Supported delimiter character.
     */
    export type HthvDelimiterChar = ' ' | '\t' | ',' | ';' | '"' | '\\' | '(' | ')' | '/' | ':' | '<' | '=' | '>' | '?' | '@' | '[' | ']' | '{' | '}';
}

declare module "@hatsy/http-header-value" {
    /**
     * Item value type.
     *
     * This is one of:
     * - `quoted-string` - A [quoted-string].
     *   > `ETag:` __`"0815"`__
     *
     *   Item {@link HthvItem.v value} is unquoted and unescaped for this type.
     * - `tagged-string` - A tagged [quoted-string]:
     *   > `ETag:` __`W/"0815"`__
     *
     *   Here `W/` is a {@link HthvItem.t tag}, while `0815` is {@link HthvItem.v unquoted value}.
     * - `angle-bracketed-string` - A string in angle brackets. This is typically used for URLs:
     *   > `Link:` __`<https://example.com/index.html?mode=preconnect>; rel="preconnect"`__
     *
     *   Item {@link HthvItem.v value} is a string with angle brackets removed. Escape sequences are not supported inside
     *   angle brackets.
     * - `date-time` - A [date-time] value in [IMF-fixdate] format.
     *   Available only when {@link HthvParserConfig.dateTime explicitly enabled}.
     *   > `Date:` __`Sun, 06 Nov 1994 08:49:37 GMT`__
     *
     * - `raw` - This type is used unless there is a more specific representation.
     *
     * [quoted-string]: https://tools.ietf.org/html/rfc7230#section-3.2.6
     * [IMF-fixdate]: https://tools.ietf.org/html/rfc7231#section-7.1.1.1
     * [date-time]: https://tools.ietf.org/html/rfc7231#section-7.1.1.1
     */
    export type HthvItemType = 'raw' | 'quoted-string' | 'tagged-string' | 'angle-bracketed-string' | 'date-time';
    /**
     * Parsed HTTP header value item.
     *
     * HTTP header value may consist of multiple such items, either comma- or space- separated.
     *
     * Some items may be nested inside another ones. E.g. as item parameters, or as nested comments.
     *
     * Item may represent a `<name>=<value>` pair used as top level item or its parameter, or a `<name>:<value>` pair within
     * comment.
     *
     * @typeParam TNameMode - Whether this item has a {@link n name}.
     * @typeParam TTagMode - Whether this item has a {@link t tag}.
     * @typeParam TParamsMode - Whether this item has {@link p parameters}.
     */
    export interface HthvItem<TNameMode extends 'has-name' | 'no-name' = 'has-name' | 'no-name', TTagMode extends 'has-tag' | 'no-tag' = 'has-tag' | 'no-tag', TParamsMode extends 'has-params' | 'no-params' = 'has-params' | 'no-params'> {
        /**
         * Item value type.
         */
        $: 'has-tag' extends TTagMode ? HthvItemType : Exclude<HthvItemType, 'tagged-string'>;
        /**
         * A name of name/value item.
         *
         * This is always a [token].
         *
         * [token]: https://tools.ietf.org/html/rfc7230#section-3.2.6
         */
        n: 'has-name' extends TNameMode ? 'no-name' extends TNameMode ? string | undefined : string : undefined;
        /**
         * A tag of tagged string.
         *
         * This is only set for `tagged-string` item type.
         */
        t: 'has-tag' extends TTagMode ? 'no-tag' extends TTagMode ? string | undefined : string : undefined;
        /**
         * Item value.
         */
        v: string;
        /**
         * Any extra items immediately following item value and preceding its parameters. Typically empty.
         */
        x: HthvExtraItem[];
        /**
         * A map of item parameters.
         *
         * Includes an entry for this item if it has a {@link n name}.
         */
        p: 'has-params' extends TParamsMode ? HthvParamMap : {
            [name: string]: never;
        };
        /**
         * A list of all item parameters.
         */
        pl: 'has-params' extends TParamsMode ? HthvParamItem[] : [
        ];
    }
    /**
     * An extra item following a value of another one and preceding its parameters.
     *
     * Extra items typically not present within standard headers, but may present in custom ones:
     * > `Calculate`: __`"foo"+"bar";foo=2;bar=3`__
     *
     * Here `"a"` is a quoted string item, `+` and `"b"` - its extra items, while `foo=2` and `bar=3` its parameters.
     */
    export type HthvExtraItem = HthvItem<'no-name', 'no-tag', 'no-params'>;
    /**
     * A map of parameters of HTTP header value item.
     *
     * Parameters are semicolon-separated items following parameterized item's value.
     *
     * This is an object literal with parameter names as keys and corresponding parameter items as values.
     *
     * When parameter has no name, then its value is used as a key.
     *
     * When multiple parameters have the same key, the first one is preferred, and the one with the name is always
     * preferred over the one without it.
     */
    export interface HthvParamMap {
        [name: string]: HthvParamItem;
    }
    /**
     * A parameter of HTTP header item.
     *
     * Parameters are semicolon-separated items following main item value. This is typically either a
     * name/value pair, or just a value. But technically can be a quoted string too. But ever a tagged string.
     *
     * > `Set-Cookie:` __`id=a3fWa; Secure; Domain=example.com; Max-Age=2592000`__
     */
    export type HthvParamItem = HthvItem<'has-name' | 'no-name', 'no-tag', 'no-params'>;
}

declare module "@hatsy/http-header-value" {
    /**
     * HTTP header value parser signature.
     *
     * Splits the value onto {@link HthvItem items}.
     *
     * @typeParam TNameMode - Whether parsed items have {@link HthvItem.n names}.
     * @typeParam TTagMode - Whether parsed items have {@link HthvItem.t tags}.
     * @typeParam TParamsMode - Whether parsed items have {@link HthvItem.p parameters}.
     */
    export type HthvParser<TNameMode extends 'has-name' | 'no-name' = 'has-name' | 'no-name', TTagMode extends 'has-tag' | 'no-tag' = 'has-tag' | 'no-tag', TParamsMode extends 'has-params' | 'no-params' = 'has-params' | 'no-params'> = 
    /**
     * @param value - HTTP header value to parse.
     *
     * @returns An array of parsed value items.
     */
    (this: void, value: string) => HthvItem<TNameMode, TTagMode, TParamsMode>[];
    /**
     * A configuration of HTTP header value parser.
     *
     * This is used to {@link newHthvParser configure new parser}.
     */
    export interface HthvParserConfig {
        /**
         * Delimits configuration.
         *
         * An object literal with {@link HthvDelimiterChar supported delimiter characters} as keys
         * and corresponding {@link HthvDelimiter delimiters} as values.
         */
        readonly delimit?: HthvDelimitConfig | undefined;
        /**
         * Whether to parse date/time values.
         *
         * @default `false`
         */
        readonly dateTime?: boolean | undefined;
        /**
         * Whether to parse comments.
         *
         * @default `false`.
         */
        readonly comments?: boolean | undefined;
    }
    /**
     * Delimits configuration for HTTP header value parser.
     *
     * This is an object literal with {@link HthvDelimiterChar supported delimiter characters} as keys
     * and corresponding {@link HthvDelimiter delimiters} as values.
     *
     * The default delimiter configuration is used for omitted characters.
     */
    export type HthvDelimitConfig = {
        readonly [char in HthvDelimiterChar]?: number | undefined;
    };
    /**
     * Creates and configures new HTTP header value parser.
     *
     * @param config - New parser configuration.
     *
     * @returns New HTTP header value parser function.
     */
    export function newHthvParser(config?: HthvParserConfig): HthvParser;
}

declare module "@hatsy/http-header-value" {
    /**
     * Parses HTTP header value that may contain comments.
     *
     * This is particularly useful when parsing headers such as:
     * - `User-Agent`
     * - `Server`
     */
    export const hthvParseCommented: HthvParser;
}

declare module "@hatsy/http-header-value" {
    /**
     * Parses HTTP header values containing directives.
     *
     * Directives are parameterized items using spaces instead of semicolons to delimit parameters. While semicolons and
     * colons are used to delimit items.
     *
     * Date/time values within directives are usually enclosed into quotes. So, no need to parse them in a special way.
     *
     * This is particularly useful when parsing headers such as:
     * - `Content-Security-Policy`
     * - `Content-Security-Policy-Report-Only`
     * - `Feature-Policy`
     * - `Via`
     * - `Warning`
     *
     * @param value - HTTP header value to parse.
     *
     * @returns An array of comma-separated value items with space-separated parameters.
     */
    export const hthvParseDirectives: HthvParser;
}

declare module "@hatsy/http-header-value" {
    /**
     * Parses semicolon-separated HTTP header value items.
     *
     * Treats ';' just like ','.
     *
     * This is particularly useful when parsing a `Cookies` header.
     *
     * The parsed items won't have any parameters.
     */
    export const hthvParseSemiSep: HthvParser<'has-name' | 'no-name', 'has-tag' | 'no-tag', 'no-params'>;
}

declare module "@hatsy/http-header-value" {
    /**
     * Parses HTTP header value containing URI(s) without parameters.
     *
     * `,`, `;`, '(', ')', and `=` symbols can be part of URI. This function returns multiple items only if URIs are
     * space-separated.
     *
     * This is particularly useful when parsing headers such as:
     * - `Content-Location`
     * - `Location`
     * - `Referef`
     *
     * @param value - HTTP header value to parse.
     *
     * @returns An array of space-separated value items without parameters.
     */
    export const hthvParseURIs: HthvParser<'no-name', 'no-tag', 'no-params'>;
}

declare module "@hatsy/http-header-value" {
    /**
     * Escapes a `string` to be included into [quoted-string](https://tools.ietf.org/html/rfc7230#section-3.2.6) within HTTP
     * header value.
     *
     * Replaces `\` with `\\`, and `"` with `\"`.
     *
     * @param string - A string to escape.
     *
     * @returns Escaped `string`.
     */
    export function hthvEscapeQ(string: string): string;
    /**
     * Escapes a `string` to be included into [comment](https://tools.ietf.org/html/rfc7230#section-3.2.6) withing HTTP
     * header value.
     *
     * Replaces `\` with `\\`, `"` with `\"`, `(` with `\(`, and `)` with `\)`.
     *
     * According to [RFC7230] the sender should not escape a double quote in this case. However, it may be necessary
     * to distinguish between raw double quote and nested [quoted-string].
     *
     * [RFC7230]: https://tools.ietf.org/html/rfc7230
     * [quoted-string]: https://tools.ietf.org/html/rfc7230#section-3.2.6
     *
     * @param string - A string to escape.
     *
     * @returns Escaped `string`.
     */
    export function hthvEscapeC(string: string): string;
}

declare module "@hatsy/http-header-value" {
    /**
     * HTTP header value items collection.
     *
     * Contains a map of named items, as well as their list.
     *
     * This is constructed by {@link hthvFlatten} function.
     *
     * @typeParam TNameMode - Whether items has a {@link HthvItem#n name}.
     * @typeParam TTagMode - Whether items has a {@link HthvItem#t tag}.
     * @typeParam TParamsMode - Whether items has {@link HthvItem#p parameters}.
     */
    export interface HthvItems<TNameMode extends 'has-name' | 'no-name' = 'has-name' | 'no-name', TTagMode extends 'has-tag' | 'no-tag' = 'has-tag' | 'no-tag', TParamsMode extends 'has-params' | 'no-params' = 'has-params' | 'no-params'> {
        /**
         * A map of HTTP header value items.
         *
         * When item has no name, then its value is used as a key.
         */
        map: {
            [name: string]: HthvItem<TNameMode, TTagMode, TParamsMode>;
        };
        /**
         * A list of items and their parameters.
         */
        list: HthvItem<TNameMode, TTagMode, TParamsMode>[];
    }
    /**
     * Flattens HTTP header value items by extracting their parameters.
     *
     * The result is an items collection containing original `items`, as well as their parameters.
     *
     * Recursively places `items` and their parameters to result map in their original order, and:
     * - prefers named items over unnamed ones,
     * - prefers original items over their parameters,
     * - prefers items added first.
     *
     * @typeParam TNameMode - Whether items has a {@link HthvItem#n name}.
     * @typeParam TTagMode - Whether items has a {@link HthvItem#t tag}.
     * @typeParam TParamsMode - Whether items has {@link HthvItem#p parameters}.
     * @param items Items collection.
     */
    export function hthvFlatten<TNameMode extends 'has-name' | 'no-name', TTagMode extends 'has-tag' | 'no-tag', TParamsMode extends 'has-params' | 'no-params'>(items: HthvItem<TNameMode, TTagMode, TParamsMode>[]): HthvItems<TNameMode, TTagMode, TParamsMode>;
}

declare module "@hatsy/http-header-value" {
    /**
     * Parses HTTP header value.
     *
     * Splits the value onto {@link HthvItem items}.
     *
     * Parser is configured with default {@link HthvParserConfig config}.
     *
     * Does not recognize comments or date/time values.
     *
     * Treats illegal characters as ASCII letters.
     *
     * @param value - HTTP header value to parse.
     *
     * @returns An array of comma- or space- separated value items.
     */
    export const hthvParse: HthvParser;
    /**
     * Parses HTTP header value that may contain date/time.
     *
     * Splits the value onto {@link HthvItem items}.
     *
     * Parser is configured with default config except {@link HthvParserConfig.dateTime date/time parsing enabled}.
     *
     * Does not recognize comments.
     *
     * Handles date/time values in [IMF-fixdate] format only.
     *
     * Treats illegal characters as ASCII letters.
     *
     * [IMF-fixdate]: https://tools.ietf.org/html/rfc7231#section-7.1.1.1
     *
     * @param value - HTTP header value to parse.
     *
     * @returns An array of comma- or space- separated value items.
     */
    export const hthvParseDT: HthvParser;
}

declare module "@hatsy/http-header-value" {
    /**
     * Conditionally encloses HTTP header value or its part into double quotes.
     *
     * Quotes will be added if delimiter or special character is present in input `string`, or the input `string` is empty.
     * {@link hthvEscapeQ Escapes} `"` and `\` symbols.
     *
     * @param string - A string to quote.
     *
     * @returns Either a string enclosed in double quotes, or original `string` if there is no need to quote it.
     */
    export function hthvQuote(string: string): string;
}
//# sourceMappingURL=http-header-value.d.ts.map
