/// <reference lib="es2019" />
/// <reference path="http-header-value.d.ts" />
declare module "@hatsy/http-header-value/headers" {
    import type { HthvItem } from "@hatsy/http-header-value";
    /**
     * HTTP content negotiator signature.
     *
     * Extracts a value matching the given HTTP content negotiation request.
     *
     * This is used to process request headers like [Accept], [Accept-Encoding], [Accept-Language], etc.
     *
     * The content negotiator can be constructed by {@link httpContentNegotiator} function.
     *
     * [Accept]: https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Accept
     * [Accept-Encoding]: https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Accept-Encoding
     * [Accept-Language]: https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Accept-Language
     *
     * @typeParam T - A type of the matching value.
     * @param request - Content negotiation request. This an `Accept...` header value, either
     * {@link @hatsy/http-header-value!hthvParse parsed}, or as a raw string.
     *
     * @returns The best matching value, `undefined` if there is no matching value, or `0` is the matching
     * value is explicitly prohibited (i.e. `;q=0` is used in matching request item).
     */
    export type HttpContentNegotiator<T> = (this: void, request: string | Iterable<HthvItem>) => T | undefined | 0;
    export namespace HttpContentNegotiator {
        /**
         * A signature of function that extracts all wildcards the given content negotiation key matches.
         *
         * @param key - Source key.
         *
         * @returns All wildcards the `key` matches, starting from the most specific one (the key itself) and ending with the
         * most generic one (i.e. a match-all wildcard).
         */
        type Wildcards = (this: void, key: string) => readonly [
            string,
            ...string[]
        ];
        /**
         * A map of values corresponding to content negotiation keys.
         *
         * @typeParam T - A type of content.
         */
        interface Map<T> {
            /**
             * Contains a value corresponding to content negotiation key used as property name.
             */
            readonly [key: string]: T;
        }
    }
    /**
     * Builds HTTP content negotiator.
     *
     * @typeParam T - A type of the matching values.
     * @param wildcards - A function extracting all possible wildcards the content negotiation key matches.
     * @param map - A map of values corresponding to content negotiation keys.
     *
     * @returns New HTTP content negotiator function.
     */
    export function httpContentNegotiator<T>(this: void, wildcards: HttpContentNegotiator.Wildcards, map: HttpContentNegotiator.Map<T>): HttpContentNegotiator<T>;
}

declare module "@hatsy/http-header-value/headers" {
    /**
     * Builds requested content encoding negotiator.
     *
     * This is used to process [Accept-Encoding] request header.
     *
     * [Accept-Encoding]: https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Accept-Encoding
     *
     * @typeParam T - A type of the matching values.
     * @param map - A map of values corresponding to content encodings or wildcard.
     *
     * @returns New requested content encoding negotiator function.
     */
    export function httpEncodingNegotiator<T>(map: HttpContentNegotiator.Map<T>): HttpContentNegotiator<T>;
}

declare module "@hatsy/http-header-value/headers" {
    import type { HthvItem } from "@hatsy/http-header-value";
    /**
     * A trust policy to HTTP proxy forwarding records.
     *
     * Defines how to treat proxy forwarding information contained in request headers.
     */
    export interface HttpForwardTrust {
        /**
         * Whether the forwarding records in HTTP request headers are trusted.
         *
         * When this information is trusted the host and protocol in request header is used as a request ones.
         *
         * This can be one of:
         * - `false` to never trust any proxies,
         * - `true` to trust all proxies,
         * - a {@link HttpForwardTrust.Checker trust checker function},
         * - a {@link HttpForwardTrust.Params trusted parameters map}.
         *
         * @defaultValue `false` which means the `Forwarded` and `X-Forwarded-...` headers won't be parsed.
         */
        readonly trusted?: boolean | HttpForwardTrust.Checker | HttpForwardTrust.Params;
        /**
         * Whether to consider `X-Forwarded-...` headers if `Forwarded` is absent.
         *
         * An {@link @hatsy/http-header-value!HthvItem}s corresponding to `Forwarded` records are constructed by these record
         * values.
         *
         * @default `true` which means these headers are processed.
         */
        readonly xForwarded?: boolean | undefined;
    }
    export namespace HttpForwardTrust {
        /**
         * Signature of function that checks whether the forwarding record value can be trusted.
         *
         * This function is called for each forwarding record in reverse order. The very last record is always trusted
         * as it contains local address info.
         *
         * @param item - An item of the `Forwarded` header value containing a record to check.
         * @param index - Reverse `item` index. I.e. the last one has `0` index.
         *
         * @returns Bitwise {@link HttpForwardTrustMask mask} of the trust.
         */
        type Checker = (this: void, item: HthvItem, index: number) => HttpForwardTrustMask;
        /**
         * A map of trusted parameters.
         *
         * The proxy forwarding record is trusted if it contains parameter present in this map, and its value present among
         * corresponding {@link Values trusted values}.
         */
        interface Params {
            readonly by?: Values | undefined;
            readonly for?: Values | undefined;
            /**
             * Maps forwarding record parameter name to trusted values.
             */
            readonly [name: string]: Values | undefined;
        }
        /**
         * A map of trusted parameter values.
         */
        interface Values {
            /**
             * Maps forwarding record parameter value to {@link Mask bitwise mask of the trust}.
             */
            readonly [value: string]: HttpForwardTrustMask | undefined;
        }
    }
    /**
     * Bitwise mask of the trust to proxy forwarding information.
     */
    export const enum HttpForwardTrustMask {
        /**
         * Do not trust mask.
         */
        DontTrust = 0,
        /**
         * Trust the checked record flag.
         */
        TrustCurrent = 1,
        /**
         * Trust a record preceding to checked one flag.
         */
        TrustPrevious = 2,
        /**
         * Always trust mask.
         */
        AlwaysTrust = 3
    }
    export const HttpForwardTrust: {
        /**
         * Builds HTTP request forwarding trust checker function by forwarding info trust policy.
         *
         * @param trust - A trust policy to proxy forwarding records.
         *
         * @returns Constructed trust predicate function.
         */
        by(this: void, trust?: HttpForwardTrust): HttpForwardTrust.Checker;
    };
}

declare module "@hatsy/http-header-value/headers" {
    import type { HthvItem } from "@hatsy/http-header-value";
    /**
     * HTTP request forwarding report.
     *
     * This information is collected from [Forwarded] and `X-Forwarded-...` request headers.
     *
     * [Forwarded]: https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Forwarded
     */
    export interface HttpForwardRep {
        readonly by: string;
        readonly for: string;
        readonly host: string;
        readonly proto: string;
        readonly [key: string]: string | undefined;
    }
    export namespace HttpForwardRep {
        /**
         * Request headers representation.
         */
        interface Headers {
            /**
             * [Forwarded](https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Forwarded) header value.
             */
            readonly forwarded?: string | Iterable<string> | undefined;
            /**
             * [X-Forwarded-For](https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/X-Forwarded-For) header value.
             */
            readonly 'x-forwarded-for'?: string | Iterable<string> | undefined;
            /**
             * [X-Forwarded-Host](https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/X-Forwarded-Host) header value.
             */
            readonly 'x-forwarded-host'?: string | Iterable<string> | undefined;
            /**
             * [X-Forwarded-Proto](https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/X-Forwarded-Proto) header value.
             */
            readonly 'x-forwarded-proto'?: string | Iterable<string> | undefined;
            /**
             * Header value with its lower-case name as property name.
             *
             * The value is either a single one or iterable.
             */
            readonly [name: string]: string | Iterable<string> | undefined;
        }
        /**
         * Forwarding info defaults.
         *
         * This is used to construct the last item in the forwarding info list that is always trusted.
         */
        interface Defaults {
            /**
             * Local address.
             */
            readonly by: string;
            /**
             * Remote address.
             */
            readonly for: string;
            /**
             * Either `Host` header or `${localAddress}:S{localPort}`.
             */
            readonly host: string;
            /**
             * Accepting protocol.
             *
             * E.g. `req.connection.encrypted ? 'https' : 'http'`.
             */
            readonly proto: string;
        }
    }
    export const HttpForwardRep: {
        /**
         * Builds trusted proxy forwarding report by proxy forwarding records.
         *
         * @param items - `Forwarded` header value items.
         * @param defaults - Forwarding info defaults.
         * @param trust - A trust policy to proxy forwarding records.
         *
         * @returns Trusted proxy forwarding report.
         */
        build(this: void, items: readonly HthvItem[], defaults: HttpForwardRep.Defaults, trust?: HttpForwardTrust): HttpForwardRep;
        /**
         * Builds trusted proxy forwarding report by HTTP request.
         *
         * @param headers - Request headers.
         * @param defaults - Forwarding info defaults.
         * @param trust - A trust policy to proxy forwarding records.
         *
         * @returns Trusted proxy forwarding report.
         */
        by(this: void, headers: HttpForwardRep.Headers, defaults: HttpForwardRep.Defaults, trust?: HttpForwardTrust): HttpForwardRep;
    };
}

declare module "@hatsy/http-header-value/headers" {
    /**
     * Builds requested language negotiator.
     *
     * This is used to process [Accept-Language] request header.
     *
     * [Accept-Language]: https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Accept-Language
     *
     * @typeParam T - A type of the matching values.
     * @param map - A map of values corresponding to language codes or wildcard.
     *
     * @returns New requested language negotiator function.
     */
    export function httpLanguageNegotiator<T>(map: HttpContentNegotiator.Map<T>): HttpContentNegotiator<T>;
}

declare module "@hatsy/http-header-value/headers" {
    /**
     * Builds requested MIME negotiator.
     *
     * This is used to process [Accept] request header.
     *
     * [Accept]: https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Accept
     *
     * @typeParam T - A type of the matching values.
     * @param map - A map of values corresponding to MIME types or wildcards.
     *
     * @returns New requested MIME negotiator function.
     */
    export function httpMimeNegotiator<T>(map: HttpContentNegotiator.Map<T>): HttpContentNegotiator<T>;
}
//# sourceMappingURL=http-header-value.headers.d.ts.map
