import { h as hthvParse } from './hthv-parse-89a10775.js';
export { H as HttpForwardRep, a as HttpForwardTrust } from './http-forward-rep-51e650c0.js';

/**
 * Builds HTTP content negotiator.
 *
 * @typeParam T - A type of the matching values.
 * @param wildcards - A function extracting all possible wildcards the content negotiation key matches.
 * @param map - A map of values corresponding to content negotiation keys.
 *
 * @returns New HTTP content negotiator function.
 */
function httpContentNegotiator(wildcards, map) {
    const negotiationMap = httpContentNegotiationMap(wildcards, map);
    return request => httpContentNegotiation(negotiationMap, typeof request === 'string' ? hthvParse(request) : request);
}
/**
 * @internal
 */
function httpContentNegotiationMap(wildcards, map) {
    const result = new Map();
    for (const [key, value] of Object.entries(map)) {
        const keyWildcards = wildcards(key);
        for (const wildcard of keyWildcards) {
            const prevEntry = result.get(wildcard);
            if (!prevEntry || keyWildcards.length < prevEntry[1].length) {
                result.set(wildcard, [value, keyWildcards]);
            }
        }
    }
    return result;
}
/**
 * @internal
 */
function httpContentNegotiation(map, request) {
    const candidates = [];
    const qFactor = httpContentNegotiationCandidates(map, request, candidates);
    if (!candidates.length) {
        return;
    }
    if (!qFactor) {
        return 0;
    }
    let bestCandidate = candidates[0];
    for (let i = 1; i < candidates.length; ++i) {
        const candidate = candidates[i];
        if (candidate[1].length > bestCandidate[1].length) {
            bestCandidate = candidate;
        }
    }
    return bestCandidate[0];
}
/**
 * @internal
 */
function httpContentNegotiationCandidates(map, request, candidates) {
    let qFactor = 0;
    for (const item of request) {
        const candidate = map.get(item.v);
        if (!candidate) {
            continue;
        }
        const q = httpContentNegotiationQFactor(item);
        if (q < qFactor) {
            continue;
        }
        if (q > qFactor) {
            candidates.length = 0;
            qFactor = q;
        }
        candidates.push(candidate);
    }
    return qFactor;
}
/**
 * @internal
 */
function httpContentNegotiationQFactor({ p: { q } }) {
    const str = q && q.v;
    if (!str) {
        return 1;
    }
    const value = parseFloat(str);
    return isNaN(value) || value > 1 ? 1 : Math.max(value, 0);
}

/**
 * Builds requested content encoding negotiator.
 *
 * This is used to process [Accept-Encoding] request header.
 *
 * [Accept-Encoding]: https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Accept-Encoding
 *
 * @typeParam T - A type of the matching values.
 * @param map - A map of values corresponding to content encodings or wildcard.
 *
 * @returns New requested content encoding negotiator function.
 */
function httpEncodingNegotiator(map) {
    return httpContentNegotiator(httpEncodingNegotiationWildcards, map);
}
/**
 * @internal
 */
const httpEncodingWildcard = '*';
/**
 * @internal
 */
const httpEncodingWildcards = [httpEncodingWildcard];
/**
 * @internal
 */
function httpEncodingNegotiationWildcards(key) {
    return key === httpEncodingWildcard ? httpEncodingWildcards : [key, httpEncodingWildcard];
}

/**
 * Builds requested language negotiator.
 *
 * This is used to process [Accept-Language] request header.
 *
 * [Accept-Language]: https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Accept-Language
 *
 * @typeParam T - A type of the matching values.
 * @param map - A map of values corresponding to language codes or wildcard.
 *
 * @returns New requested language negotiator function.
 */
function httpLanguageNegotiator(map) {
    return httpContentNegotiator(httpLanguageNegotiationWildcards, map);
}
/**
 * @internal
 */
const httpLanguageWildcard = '*';
/**
 * @internal
 */
const httpLanguageWildcards = [httpLanguageWildcard];
/**
 * @internal
 */
function httpLanguageNegotiationWildcards(key) {
    if (key === httpLanguageWildcard) {
        return httpLanguageWildcards;
    }
    const wildcards = [key];
    for (;;) {
        const idx = key.lastIndexOf('-');
        if (idx < 0) {
            wildcards.push(key, httpLanguageWildcard);
            break;
        }
        key = key.substr(0, idx);
        wildcards.push(key);
    }
    return wildcards;
}

/**
 * Builds requested MIME negotiator.
 *
 * This is used to process [Accept] request header.
 *
 * [Accept]: https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Accept
 *
 * @typeParam T - A type of the matching values.
 * @param map - A map of values corresponding to MIME types or wildcards.
 *
 * @returns New requested MIME negotiator function.
 */
function httpMimeNegotiator(map) {
    return httpContentNegotiator(httpMimeNegotiationWildcards, map);
}
/**
 * @internal
 */
const httpMimeWildcard = '*/*';
/**
 * @internal
 */
const httpMimeWildcards = [httpMimeWildcard];
/**
 * @internal
 */
function httpMimeNegotiationWildcards(key) {
    if (key.endsWith('/*')) {
        return key === httpMimeWildcard ? httpMimeWildcards : [key, httpMimeWildcard];
    }
    const slashIdx = key.indexOf('/');
    if (slashIdx < 0) {
        return [`${key}/*`, httpMimeWildcard];
    }
    const type = key.substr(0, slashIdx);
    return [key, type + '/*', httpMimeWildcard];
}

export { httpContentNegotiator, httpEncodingNegotiator, httpLanguageNegotiator, httpMimeNegotiator };
//# sourceMappingURL=http-header-value.headers.js.map
