import { n as newHthvParser, H as HthvDelimiter, d as defaultParserConfig, c as commentParserConfig } from './hthv-parse-89a10775.js';
export { h as hthvParse, a as hthvParseDT } from './hthv-parse-89a10775.js';

/**
 * Parses HTTP header value that may contain comments.
 *
 * This is particularly useful when parsing headers such as:
 * - `User-Agent`
 * - `Server`
 */
const hthvParseCommented = /*#__PURE__*/ newHthvParser({ comments: true });

/**
 * Parses HTTP header values containing directives.
 *
 * Directives are parameterized items using spaces instead of semicolons to delimit parameters. While semicolons and
 * colons are used to delimit items.
 *
 * Date/time values within directives are usually enclosed into quotes. So, no need to parse them in a special way.
 *
 * This is particularly useful when parsing headers such as:
 * - `Content-Security-Policy`
 * - `Content-Security-Policy-Report-Only`
 * - `Feature-Policy`
 * - `Via`
 * - `Warning`
 *
 * @param value - HTTP header value to parse.
 *
 * @returns An array of comma-separated value items with space-separated parameters.
 */
const hthvParseDirectives = /*#__PURE__*/ newHthvParser({
    delimit: {
        ' ': HthvDelimiter.NonToken | HthvDelimiter.Space | HthvDelimiter.Parameter,
        '\t': HthvDelimiter.NonToken | HthvDelimiter.Space | HthvDelimiter.Parameter,
        ';': HthvDelimiter.NonToken | HthvDelimiter.Item,
    },
});

/**
 * Parses semicolon-separated HTTP header value items.
 *
 * Treats ';' just like ','.
 *
 * This is particularly useful when parsing a `Cookies` header.
 *
 * The parsed items won't have any parameters.
 */
const hthvParseSemiSep = /*#__PURE__*/ newHthvParser({
    delimit: {
        ';': HthvDelimiter.NonToken | HthvDelimiter.Item,
    },
});

/**
 * Parses HTTP header value containing URI(s) without parameters.
 *
 * `,`, `;`, '(', ')', and `=` symbols can be part of URI. This function returns multiple items only if URIs are
 * space-separated.
 *
 * This is particularly useful when parsing headers such as:
 * - `Content-Location`
 * - `Location`
 * - `Referef`
 *
 * @param value - HTTP header value to parse.
 *
 * @returns An array of space-separated value items without parameters.
 */
const hthvParseURIs = 
/*#__PURE__*/ newHthvParser({
    delimit: {
        ',': HthvDelimiter.NonToken,
        ';': HthvDelimiter.NonToken,
        '=': HthvDelimiter.NonToken,
    },
});

/**
 * Escapes a `string` to be included into [quoted-string](https://tools.ietf.org/html/rfc7230#section-3.2.6) within HTTP
 * header value.
 *
 * Replaces `\` with `\\`, and `"` with `\"`.
 *
 * @param string - A string to escape.
 *
 * @returns Escaped `string`.
 */
function hthvEscapeQ(string) {
    return hthvEscape(string, defaultParserConfig);
}
/**
 * Escapes a `string` to be included into [comment](https://tools.ietf.org/html/rfc7230#section-3.2.6) withing HTTP
 * header value.
 *
 * Replaces `\` with `\\`, `"` with `\"`, `(` with `\(`, and `)` with `\)`.
 *
 * According to [RFC7230] the sender should not escape a double quote in this case. However, it may be necessary
 * to distinguish between raw double quote and nested [quoted-string].
 *
 * [RFC7230]: https://tools.ietf.org/html/rfc7230
 * [quoted-string]: https://tools.ietf.org/html/rfc7230#section-3.2.6
 *
 * @param string - A string to escape.
 *
 * @returns Escaped `string`.
 */
function hthvEscapeC(string) {
    return hthvEscape(string, commentParserConfig);
}
/**
 * @internal
 */
function hthvEscape(string, { delimiterOf }) {
    let escaped;
    for (let i = 0; i < string.length; ++i) {
        const c = string[i];
        if (delimiterOf(c) & HthvDelimiter.Escaped) {
            if (!escaped) {
                escaped = string.substring(0, i);
            }
            escaped += '\\' + c;
        }
        else if (escaped) {
            escaped += c;
        }
    }
    return escaped || string;
}

/**
 * Flattens HTTP header value items by extracting their parameters.
 *
 * The result is an items collection containing original `items`, as well as their parameters.
 *
 * Recursively places `items` and their parameters to result map in their original order, and:
 * - prefers named items over unnamed ones,
 * - prefers original items over their parameters,
 * - prefers items added first.
 *
 * @typeParam TNameMode - Whether items has a {@link HthvItem#n name}.
 * @typeParam TTagMode - Whether items has a {@link HthvItem#t tag}.
 * @typeParam TParamsMode - Whether items has {@link HthvItem#p parameters}.
 * @param items Items collection.
 */
function hthvFlatten(items) {
    const map = {};
    const list = [];
    const depths = {};
    const put = (item, depth) => {
        // eslint-disable-next-line @typescript-eslint/no-unsafe-argument
        list.push(item);
        const key = item.n || item.v;
        const prev = map[key];
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        if (prev) {
            if (item.n) {
                if (prev.n && depths[key] <= depth) {
                    return;
                }
            }
            else if (prev.n || depths[key] <= depth) {
                return;
            }
        }
        map[key] = item;
        depths[key] = depth;
    };
    const add = (item, depth) => {
        put(item, depth);
        ++depth;
        item.pl.forEach(p => add(p, depth));
    };
    items.forEach(item => add(item, 0));
    return { map, list };
}

/**
 * Conditionally encloses HTTP header value or its part into double quotes.
 *
 * Quotes will be added if delimiter or special character is present in input `string`, or the input `string` is empty.
 * {@link hthvEscapeQ Escapes} `"` and `\` symbols.
 *
 * @param string - A string to quote.
 *
 * @returns Either a string enclosed in double quotes, or original `string` if there is no need to quote it.
 */
function hthvQuote(string) {
    if (!string) {
        return '""';
    }
    const { delimiterOf } = defaultParserConfig;
    let escaped;
    let quote = false;
    for (let i = 0; i < string.length; ++i) {
        const c = string[i];
        const delimiter = delimiterOf(c);
        if (delimiter) {
            if (delimiter & HthvDelimiter.Escaped) {
                if (!escaped) {
                    escaped = string.substring(0, i);
                }
                escaped += '\\' + c;
            }
            quote = true;
        }
        else if (escaped) {
            escaped += c;
        }
    }
    return quote ? `"${escaped || string}"` : string;
}

export { HthvDelimiter, hthvEscapeC, hthvEscapeQ, hthvFlatten, hthvParseCommented, hthvParseDirectives, hthvParseSemiSep, hthvParseURIs, hthvQuote, newHthvParser };
//# sourceMappingURL=http-header-value.js.map
